@@ -1,10 +1,8 @@
 ;; Forth mode for Emacs
-;; This file is part of GForth.
-;; Changes by anton
-;; This is a variant of forth.el that came with TILE.
-;; I left most of this stuff untouched and made just a few changes for 
-;; the things I use (mainly indentation and syntax tables).
-;; So there is still a lot of work to do to adapt this to gforth.
+
+;; Copyright (C) 1995,1996,1997,1998 Free Software Foundation, Inc.
+
+;; This file is part of Gforth.
 
 ;; GForth is distributed in the hope that it will be useful,
 ;; but WITHOUT ANY WARRANTY.  No author or distributor
@@ -21,6 +19,12 @@
 ;; file named COPYING.  Among other things, the copyright notice
 ;; and this notice must be preserved on all copies.
 
+;; Changes by anton
+;; This is a variant of forth.el that came with TILE.
+;; I left most of this stuff untouched and made just a few changes for 
+;; the things I use (mainly indentation and syntax tables).
+;; So there is still a lot of work to do to adapt this to gforth.
+
 ;;-------------------------------------------------------------------
 ;; A Forth indentation, documentation search and interaction library
 ;;-------------------------------------------------------------------
@@ -36,7 +40,7 @@
 
 
 (defvar forth-positives
-  " : :noname code interpretation: ;code does> begin do ?do +do -do u+do u-do while if ?dup-if ?dup-0=-if else case of struct [if] [else] with public: private: class "
+  " : :noname code interpretation: ;code does> begin do ?do +do -do u+do u-do while if ?dup-if ?dup-0=-if else case of struct [if] [ifdef] [ifundef] [else] with public: private: class "
   "Contains all words which will cause the indent-level to be incremented
 on the next line.
 OBS! All words in forth-positives must be surrounded by spaces.")
@@ -48,13 +52,13 @@ on the current line.
 OBS! All words in forth-negatives must be surrounded by spaces.")
 
 (defvar forth-zeroes
-  " : :noname code interpretation: public: private: how: class class; "
+  " : :noname code interpretation: public: private: how: implements class class; "
   "Contains all words which causes the indent to go to zero")
 
 (setq forth-zero 0)
 
 (defvar forth-zup
-  " how: "
+  " how: implements "
   "Contains all words which causes zero indent level to change")
 
 (defvar forth-zdown
@@ -76,10 +80,6 @@ OBS! All words in forth-negatives must be surrounded by spaces.")
 (if (not forth-mode-map)
     (setq forth-mode-map (make-sparse-keymap)))
 
-(global-set-key "\e\C-m" 'forth-send-paragraph)
-(global-set-key "\C-x\C-m" 'forth-split)
-(global-set-key "\e " 'forth-reload)
-
 ;(define-key forth-mode-map "\M-\C-x" 'compile)
 (define-key forth-mode-map "\C-x\\" 'comment-region)
 (define-key forth-mode-map "\C-x~" 'forth-remove-tracers)
@@ -92,7 +92,7 @@ OBS! All words in forth-negatives must be surrounded by spaces.")
 (define-key forth-mode-map "\M-q" 'forth-fill-paragraph)
 (define-key forth-mode-map "\e." 'forth-find-tag)
 
-(load "etags.el")
+(load "etags")
 
 (defun forth-find-tag (tagname &optional next-p regexp-p)
   (interactive (find-tag-interactive "Find tag: "))
@@ -160,7 +160,7 @@ OBS! All words in forth-negatives must be surrounded by spaces.")
   "
 Major mode for editing Forth code. Tab indents for Forth code. Comments
 are delimited with \\ and newline. Paragraphs are separated by blank lines
-only. Delete converts tabs to spaces as it moves back.
+only.
 \\{forth-mode-map}
  Forth-split
     Positions the current buffer on top and a forth-interaction window
@@ -421,6 +421,10 @@ part of the screen."
   (forth-split)
   (forth-set-runlight forth-runlight:input))
 
+(defun run-forth-if-not ()
+  (if (not (forth-process-running-p))
+      (run-forth forth-program-name)))
+
 (defun reset-forth ()
   "Reset the Forth process."
   (interactive)
@@ -714,7 +718,7 @@ The region is sent terminated by a newline."
   (interactive)
   (let ((process (get-process forth-program-name)))
     (if process (kill-process process t)))
-  (sleep-for-millisecs 100)
+  (sleep-for 0 100)
   (forth-mode))
 
 
@@ -922,18 +926,44 @@ The region is sent terminated by a newline."
 
 (require 'outline)
 
-;;(define-key outline-minor-mode-map 'f9 'show-entry)
-;;(define-key outline-minor-mode-map 'f10 'hide-entry)
+(defun f-outline-level ()
+	(cond	((looking-at "\\`\\\\")
+			0)
+		((looking-at "\\\\ SEC")
+			0)
+		((looking-at "\\\\ \\\\ .*")
+			0)
+		((looking-at "\\\\ DEFS")
+			1)
+		((looking-at "\\/\\* ")
+			1)
+		((looking-at ": .*")
+			1)
+		((looking-at "\\\\G")
+			2)
+		((looking-at "[ \t]+\\\\")
+			3))
+)			
 
 (defun fold-f  ()
    (interactive)
    (add-hook 'outline-minor-mode-hook 'hide-body)
 
    ; outline mode header start, i.e. find word definitions
-   (setq  outline-regexp  "^\\(:\\)[ \t]+\\([^ \t]*\\)")
+;;;   (setq  outline-regexp  "^\\(:\\)[ \t]+\\([^ \t]*\\)")
+   (setq  outline-regexp  "\\`\\\\\\|:\\|\\\\ SEC\\|\\\\G\\|[ \t]+\\\\\\|\\\\ DEFS\\|\\/\\*\\|\\\\ \\\\ .*")
+   (setq outline-level 'f-outline-level)
 
    (outline-minor-mode)
+   (define-key outline-minor-mode-map '(shift up) 'hide-sublevels)
+   (define-key outline-minor-mode-map '(shift right) 'show-children)
+   (define-key outline-minor-mode-map '(shift left) 'hide-subtree)
+   (define-key outline-minor-mode-map '(shift down) 'show-subtree)
+
 )
+
+;;(define-key global-map '(shift up) 'fold-f)
+
 ;;; end file folding
 
 ;;; func-menu is a package that scans your source file for function definitions
@@ -946,6 +976,9 @@ The region is sent terminated by a newline."
        (require 'func-menu)
 ;;       (define-key global-map 'f8 'function-menu)
        (add-hook 'find-fible-hooks 'fume-add-menubar-entry)
-       (define-key global-map "\C-cg" 'fume-prompt-function-goto)
-       (define-key global-map '(shift button3) 'mouse-function-menu)
-       ))
+;       (define-key global-map "\C-cg" 'fume-prompt-function-goto)
+;       (define-key global-map '(shift button3) 'mouse-function-menu)
+))
+
+;; end
+
